unit EnUS;

{
EnUS

Author : Serdar S. Kacar
Company: BTT Ltd. Co.
Date   : Oct 5, 2010
License: Free for use/modify/distribute

Remarks:

Programs that store variables as strings by various formating functions
  have a serious risk of unexpected behaviour in the following cases :
1. Variables written in a locale and read from another one,
2. Even if the locales are the same, a user has "customized" her own locale
   settings.

Something as simple as storing a double in an INI file could be bothersome.

The solution is already, but partially, provided by Delphi as various formating
  functions support an optional a parameter of type TFormatSettings.

A constant of type TFormatSettings that "never" changes is the only thing we
  need to satisfy the locale-safe requirements.

This unit includes English (United States) format settings as
  const FormatSettingsEnUS: TFormatSettings = (...)

This unit also includes conversion routines for TDateTime and ctime() formatted
  strings.

Keywords: Localization, Format, FormatFloat, DecimalSeparator, ctime()
}

interface

uses SysUtils;

const FormatSettingsEnUS: TFormatSettings = (
  CurrencyFormat        : 0; // Prefix, no separation, for example $1.1
  NegCurrFormat         : 0; // Left parenthesis,monetary symbol,number,right parenthesis. Example: ($1.1)
  ThousandSeparator     : ',';
  DecimalSeparator      : '.';
  CurrencyDecimals      : 2;
  DateSeparator         : '/';
  TimeSeparator         : ':';
  ListSeparator         : ',';
  CurrencyString        : '$';
  ShortDateFormat       : 'M/d/yyyy';
  LongDateFormat        : 'dddd, MMMM dd, yyyy';
  TimeAMString          : 'AM';
  TimePMString          : 'PM';
  ShortTimeFormat       : 'h:mm AMPM';
  LongTimeFormat        : 'h:mm:ss AMPM';
  ShortMonthNames : (
    'Jan',
    'Feb',
    'Mar',
    'Apr',
    'May',
    'Jun',
    'Jul',
    'Aug',
    'Sep',
    'Oct',
    'Nov',
    'Dec'
    );
  LongMonthNames : (
    'January',
    'February',
    'March',
    'April',
    'May',
    'June',
    'July',
    'August',
    'September',
    'October',
    'November',
    'December'
    );
  ShortDayNames : (
    'Sun',
    'Mon',
    'Tue',
    'Wed',
    'Thu',
    'Fri',
    'Sat'
    );
  LongDayNames : (
    'Sunday',
    'Monday',
    'Tuesday',
    'Wednesday',
    'Thursday',
    'Friday',
    'Saturday'
    );
  );


{ ctime()
  The string result produced by ctime is in "local time" by definition.
  The string result produced by ctime contains exactly 26 characters and has the form:
    Wed Jan 02 02:03:55 1980\n\0  }
const DTFORMAT_CTIME_CORE = 'ddd mmm dd hh:nn:ss yyyy';
const DTFORMAT_CTIME_SUFFIX = #10#0;
function FormatDTLocalAs_ctime(const AValue: TDateTime; const bIncludeNewLineAndNULLSuffix: Boolean): AnsiString;
function ctimeToDTLocal(const AValue: AnsiString): TDateTime;


implementation

uses DateUtils, RTLConsts, SysConst;

function FormatDTLocalAs_ctime(const AValue: TDateTime; const bIncludeNewLineAndNULLSuffix: Boolean): AnsiString;
begin
  Result := FormatDateTime(DTFORMAT_CTIME_CORE, AValue, FormatSettingsEnUS);
  if bIncludeNewLineAndNULLSuffix then Result := Result + DTFORMAT_CTIME_SUFFIX;
end;

function ctimeToDTLocal(const AValue: AnsiString): TDateTime;
var
  i: Integer; ShortMonthName: AnsiString;
  AYear, AMonth, ADay, AHour, AMinute, ASecond: Word;
begin
  if Length(AValue) < Length(DTFORMAT_CTIME_CORE) then raise EConvertError.CreateFmt(SysConst.SInvalidDateTime,[AValue]);

  ShortMonthName := Copy(AValue, 5, 3);
  AMonth := 0;
  for i := Low(FormatSettingsEnUS.ShortMonthNames) to High(FormatSettingsEnUS.ShortMonthNames) do
  begin
    if AnsiSameText(ShortMonthName, FormatSettingsEnUS.ShortMonthNames[i]) then
    begin
      AMonth := i;
      break;
    end;
  end;
  if AMonth = 0 then raise EConvertError.CreateFmt(SysConst.SInvalidDateTime,[AValue]);

  ADay    := StrToInt(Copy(AValue,  9, 2));
  AHour   := StrToInt(Copy(AValue, 12, 2));
  AMinute := StrToInt(Copy(AValue, 15, 2));
  ASecond := StrToInt(Copy(AValue, 18, 2));
  AYear   := StrToInt(Copy(AValue, 21, 4));
  
  Result := EncodeDateTime(AYear, AMonth, ADay, AHour, AMinute, ASecond, 0);
end;

end.
